import 'package:adeptutils/adeptutils.dart';
import 'package:graphql_flutter/graphql_flutter.dart';
import 'package:graphql_flutter/graphql_flutter.dart' hide JsonSerializable;

/// This class works as abstraction for GraphQL errors by parsing error to Object

class GraphQLBaseError extends Jsonable<GraphQLBaseError> {
  static List<ErrorLocation> errorLocationListFromJson(dynamic json) {
    if (json is List<ErrorLocation>) return json;
    List<ErrorLocation> list = [];
    if (json is List<Map<String, dynamic>>) {
      for (var map in json) {
        int? line = int.tryParse(map['line'].toString());
        int? column = int.tryParse(map['column'].toString());
        if (line != null && column != null) {
          list.add(ErrorLocation(line: line, column: column));
        }
      }
    }
    return list;
  }

  static List<Map<String, dynamic>>? errorLocationListToJson(
      List<ErrorLocation>? list) {
    return list
        ?.map((value) => {
              'line': value.line,
              'column': value.column,
            })
        .toList();
  }

  String? message;
  @JsonKey(toJson: errorLocationListToJson, fromJson: errorLocationListFromJson)
  List<ErrorLocation>? locations;
  List<dynamic>? path;
  GraphQLBaseExtension? extensions;

  GraphQLBaseError({this.message, this.locations, this.path, this.extensions});

  String? get extensionsCode => extensions?.code;
  String? get responseCode => extensions?.exception?.response?.code;
  int? get exceptionStatus => extensions?.exception?.status;
  int? get responseStatus => extensions?.exception?.response?.status;

  @override
  Map<String, dynamic> toJson() => <String, dynamic>{
        'message': message,
        'locations': locations,
        'path': path,
        'extensions': extensions,
      };
  @override
  GraphQLBaseError? fromJsonMap(Map<String, dynamic>? json) =>
      json != null ? GraphQLBaseError.fromJson(json) : null;
  factory GraphQLBaseError.fromJson(Map<String, dynamic> json) =>
      GraphQLBaseError(
        message: json['message'] as String?,
        locations: errorLocationListFromJson(json['locations']),
        path: json['path'] as List<dynamic>?,
        extensions: json['extensions'] == null
            ? null
            : GraphQLBaseExtension.fromJson(
                json['extensions'] as Map<String, dynamic>),
      );
}

class GraphQLBaseExtension extends Jsonable<GraphQLBaseExtension> {
  String? code;
  GraphQLBaseException? exception;

  GraphQLBaseExtension({this.code, this.exception});

  @override
  Map<String, dynamic> toJson() => <String, dynamic>{
        'code': code,
        'exception': exception,
      };
  @override
  GraphQLBaseExtension? fromJsonMap(Map<String, dynamic>? json) =>
      json != null ? GraphQLBaseExtension.fromJson(json) : null;
  factory GraphQLBaseExtension.fromJson(Map<String, dynamic> json) =>
      GraphQLBaseExtension(
        code: json['code'] as String?,
        exception: json['exception'] == null
            ? null
            : GraphQLBaseException.fromJson(
                json['exception'] as Map<String, dynamic>),
      );
}

class GraphQLBaseException extends Jsonable<GraphQLBaseException> {
  GraphQLBaseExceptionResponse? response;
  int? status;
  String? message;
  List<dynamic>? stacktrace;

  GraphQLBaseException(
      {this.response, this.status, this.message, this.stacktrace});

  @override
  Map<String, dynamic> toJson() => <String, dynamic>{
        'response': response,
        'status': status,
        'message': message,
        'stacktrace': stacktrace,
      };
  @override
  GraphQLBaseException? fromJsonMap(Map<String, dynamic>? json) =>
      json != null ? GraphQLBaseException.fromJson(json) : null;
  factory GraphQLBaseException.fromJson(Map<String, dynamic> json) =>
      GraphQLBaseException(
        response: json['response'] == null
            ? null
            : GraphQLBaseExceptionResponse.fromJson(
                json['response'] as Map<String, dynamic>),
        status: json['status'] as int?,
        message: json['message'] as String?,
        stacktrace: json['stacktrace'] as List<dynamic>?,
      );
}

class GraphQLBaseExceptionResponse
    extends Jsonable<GraphQLBaseExceptionResponse> {
  int? status;
  String? message;
  String? code;
  String? error;

  GraphQLBaseExceptionResponse({
    this.status,
    this.message,
    this.code,
    this.error,
  });

  @override
  Map<String, dynamic> toJson() => <String, dynamic>{
        'status': status,
        'message': message,
        'code': code,
        'error': error,
      };
  @override
  GraphQLBaseExceptionResponse? fromJsonMap(Map<String, dynamic>? json) =>
      json != null ? GraphQLBaseExceptionResponse.fromJson(json) : null;
  factory GraphQLBaseExceptionResponse.fromJson(Map<String, dynamic> json) =>
      GraphQLBaseExceptionResponse(
        status: json['status'] as int?,
        message: json['message'] as String?,
        code: json['code'] as String?,
        error: json['error'] as String?,
      );
}
