import 'package:adeptutils/adeptutils.dart';
import 'package:graphql_flutter/graphql_flutter.dart';

typedef DataTypeQueryResponseConverter<T> = T Function(
    QueryResult? queryResult);

/// Used to parse data from GraphQL query responses

class QueryResponseConverter<T> {
  T? dataType;
  late DataTypeQueryResponseConverter<List<T>?> toDataListResponseConverter;
  late DataTypeQueryResponseConverter<T?> toDataResponseConverter;
  QueryResponseConverter({
    this.dataType,
  }) {
    toDataListResponseConverter = (queryResult) => toDataList(queryResult);
    toDataResponseConverter = (queryResult) => toData(queryResult);
  }

  List<T>? toDataList(QueryResult? queryResult) {
    try {
      if (queryResult?.data == null) return null;
      List? list = getResultData<List>(queryResult);
      if (dataType is Jsonable) {
        return (dataType as Jsonable).fromJsonList(list) as List<T>?;
      }
      if (list is List<T>) return list;
    } catch (e) {
      print(e);
    }
    return null;
  }

  T? toData(QueryResult? queryResult) {
    try {
      if (queryResult?.data == null) return null;
      dynamic data = queryResult!.data!.length > 1
          ? queryResult.data!.values.elementAt(1)
          : null;
      if (dataType is Jsonable) {
        return (dataType as Jsonable).fromJsonMap(data) as T?;
      }
      if (data is T) return data;
    } catch (e) {
      print(e);
    }
    return null;
  }

  static DataType? getResultData<DataType>(QueryResult? queryResult) {
    if (queryResult?.data != null) {
      for (dynamic value in queryResult!.data!.values) {
        if (value is DataType) return value;
      }
    }
    return null;
  }
}
