import 'package:adeptarch_base/adeptarch_base.dart';
import 'package:adeptgraphql/adeptgraphql.dart';

abstract class SuperSubscriptionRepository<Dao extends SuperDao<DataType>,
    DataType> {
  Dao dao;
  AdeptPublishSubject<SubscriptionResource<DataType>> publishSubject;

  SuperSubscriptionRepository(this.dao)
      : publishSubject = AdeptPublishSubject();

  AdeptPublishSubject<SubscriptionResource<DataType?>> getObservableData() =>
      publishSubject;

  /// Loading
  void postLoading(
      {DataType? resourceData,
      AdeptPublishSubject<SubscriptionResource<DataType?>>?
          publishSubject}) async {
    publishSubject ??= this.publishSubject;
    resourceData ??=
        publishSubject.value != null ? publishSubject.value!.data : null;
    publishSubject.post(SubscriptionResource<DataType>.loading(resourceData));
  }

  /// Success
  void postData(
      {DataType? resourceData,
      AdeptPublishSubject<SubscriptionResource<DataType?>>?
          publishSubject}) async {
    publishSubject ??= this.publishSubject;
    resourceData ??=
        publishSubject.value != null ? publishSubject.value!.data : null;
    publishSubject.post(SubscriptionResource<DataType>.data(resourceData));
  }

  /// Error
  void postError(String? message,
      {DataType? resourceData,
      int? code,
      dynamic exception,
      AdeptPublishSubject<SubscriptionResource<DataType?>>?
          publishSubject}) async {
    publishSubject ??= this.publishSubject;
    resourceData ??=
        publishSubject.value != null ? publishSubject.value!.data : null;
    publishSubject.post(SubscriptionResource<DataType>.error(
        resourceData, message,
        code: code, exception: exception));
  }

  /// Done
  void postDone(
      {AdeptPublishSubject<SubscriptionResource<DataType?>>?
          publishSubject}) async {
    publishSubject ??= this.publishSubject;
    publishSubject.post(SubscriptionResource<DataType>.done());
  }
}
